# numerical tokens
INT_TOKEN = "_int_"
FLOAT_TOKEN = "_float_"
INT_TOKEN_WITH_SPACE = f" {INT_TOKEN} "
FLOAT_TOKEN_WITH_SPACE = f" {FLOAT_TOKEN} "


#
START_TOKEN = "<sos>"
END_TOKEN = "<eos>"
UNK_TOKEN = "<unk>"
PAD_TOKEN = "<pad>"
